<?php

namespace App\Models;

use App\Utils\TransporteUtils;

use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use League\Csv\CannotInsertRecord;
use League\Csv\Writer;

class Barrio extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'nombre',
    ];

    /**
     * La región a la que pertenece el barrio.
     */
    public function region(): BelongsTo
    {
        return $this->belongsTo(Region::class);
    }

    /**
     * Los pedidos que pertenecen al barrio.
     */
    public function pedidos(): HasMany
    {
        return $this->hasMany(Pedido::class);
    }

    public function crearPedido(string $nombre) : Pedido {
        return $this->pedidos()->create(['nombre' => $nombre]);
    }

    public function productosPedidos() {
        return DB::table('productos')
            ->join('pedido_producto', 'productos.id', '=', 'pedido_producto.producto_id')
            ->join('pedidos', 'pedidos.id', '=', 'pedido_producto.pedido_id')
            ->where(['pedidos.barrio_id' => $this->id, 'pedidos.pagado' => true])
            ->select('productos.*', 
                     DB::raw('SUM(pedido_producto.cantidad) as cantidad'),
                     DB::raw('SUM(pedido_producto.cantidad * productos.precio) as total'))
            ->groupBy('productos.id')
            ->get();
    }

    public function totalARecaudar() : float {
        return $this->pedidos()->where(['pagado' => true])->get()->sum(
            fn($p) => $p->totalATransferir()
        );
    }

    public function totalATransferir() : float {
        return $this->totalNoBarriales() + $this->totalBonosDeTransporte();
    }

    public function totalNoBarriales() {
        return $this->totalProductosIf(fn($p) => !$p->barrial);
    }

    public function totalBarriales() {
        return $this->totalProductosIf(fn($p) => $p->barrial);
    }

    private function totalProductosIf(callable $predicado) : float {
        return $this->productosPedidos()->sum(
            function ($producto) use ($predicado) {
                return $predicado($producto) ? $producto->total : 0;
            }
        );
    }

    public function totalBonosDeTransporte() : int {
        return TransporteUtils::calcularTotal($this->totalNoBarriales());
    }

    /**
     * Los productos que pertenecen al barrio.
     */
    public function productos(): HasMany
    {
        return $this->hasMany(Producto::class);
    }

    function exportarPedidoACsv() {
        if ($this->pedidosPagados()->exists()) {
            $columnaProductos = $this->armarColumnasPedido();

            try {
                $writer = Writer::createFromPath(resource_path('csv/exports/'.$this->nombre.'.csv'), 'w');
                $writer->insertAll($columnaProductos);
            } catch (CannotInsertRecord $e) {
                var_export($e->getRecords());
            }
        }
    }

    private function armarColumnasPedido() : array {
        $columnaProductos = [];
        $filasVaciasAgregadas = false;

        foreach (Categoria::orderBy('id')->get() as $keyC => $categoria) {
            $columnaProductos[] = ['name' => $categoria->name, 'cantidad' => null];

            if ($categoria->name == 'TRANSPORTE, BONOS Y FINANCIAMIENTO SORORO')
                $columnaProductos[] = ['name' => 'Bono de Transporte', 'cantidad' => TransporteUtils::cantidad($this->totalParaTransporte)];
            else if ($categoria->name == 'PRODUCTOS DE GESTIÓN MENSTRUAL')
                $columnaProductos[] = ['name' => '¿Cuántas copas quieren y pueden comprar en el grupo?', 'cantidad' => null];

            foreach ($categoria->productos()->orderBy('id')->get() as $keyP => $producto) {
                if ($producto->price == 0 && !$filasVaciasAgregadas) {
                    $columnaProductos[] = ['name' => '¿Cuántas copas quieren adquirir a través del financiamiento sororo?', 'cantidad' => null];
                    $filasVaciasAgregadas = true;
                }
                $columnaProductos[] = ['name' => $producto->name, 'cantidad' => $this->cantidadPedida($producto->id)];
            }
        }
    }

    private function cantidadPedida($productoId) {
        $pedidos = $this->pedidos()
                        ->whereHas('productos', 
                            function ($query) use ($productoId) {
                                $query->where('producto_id', $productoId);})
                        ->get();

        return $pedidos->sum(function ($pedido) use ($productoId) {
                                return $pedido->productos
                                              ->find($productoId)
                                              ->pivot->cantidad ?? 0;});
    }
}
